from django.db import models
from io import BytesIO
from PIL import Image
from django.core.files import File
from smartfields import fields
from django.db.models.signals import pre_delete, post_save
from django.dispatch.dispatcher import receiver
from django.conf import settings
import os
from django.utils import timezone
from django.urls import reverse, reverse_lazy
from django.contrib.auth.models import User
import datetime
from django.core.validators import MaxValueValidator, MinValueValidator
future_date = (datetime.datetime.today() + datetime.timedelta(days=60)).strftime('%Y-%m-%d')

# image compression method
def compress(image):
    im = Image.open(image)
    new_width = 320
    new_height = 250
    im = im.resize((new_width, new_height), Image.ANTIALIAS)
    im_io = BytesIO()
    im.save(im_io, 'JPEG', quality=35)
    new_image = File(im_io, name=image.name)
    return new_image


def get_product_location(instance, filename):
    cat_folder = instance.subcategory.category
    subcat_folder = instance.subcategory.sub_title
    # return "{}/{}".format(instance.id, filename)
    # return "kisan_products/{catfolder}/{subcatfolder}/{id}/{pname}/{filename}".format(catfolder=cat_folder, subcatfolder=subcat_folder,id=instance.p_owner, pname=instance.p_name, filename=filename)
    return "kisan_products/{catfolder}/{id}/{filename}".format(catfolder=cat_folder, id=instance.p_owner, filename=filename)

def get_subcat_location(instance, filename, *args, **kwargs):
    cat_folder = instance.category.cat_title
    subcat_folder = instance.sub_title

    return "subcatagory/kisan_catagory/{catfolder}/{subcatfolder}/{filename}".format(catfolder=cat_folder, subcatfolder=subcat_folder, filename=filename)


class Category(models.Model):
    cat_title = models.CharField(max_length=100, db_index=True)
    cat_slug = models.SlugField(max_length=150, unique=True, db_index=True)
    cat_description = models.TextField()
    cat_image = models.ImageField(upload_to="catagoty/kisan/", blank=True, null=True)
    select_type = (
        ('KP', "Kisan"),
        ('SP', "Seller"),
    )
    cat_type = models.CharField(max_length=2, choices=select_type, default='SP')
    is_active = models.BooleanField(default=True)

    def __str__(self):
        return self.cat_title

    def get_absolute_url(self):
        return reverse("shop:category", kwargs={'slug': self.slug})


class SubCategory(models.Model):
    category = models.ForeignKey(Category, related_name='subcategories', on_delete=models.CASCADE)
    sub_title = models.CharField(max_length=150, db_index=True)
    sub_slug = models.SlugField(max_length=150, unique=True, db_index=True)
    sub_description = models.TextField()
    sub_image = models.ImageField(upload_to=get_subcat_location, blank=True, null=True)
    is_active = models.BooleanField(default=True)

    def __str__(self):
        return str(self.id) +" - "+str(self.category) +" - "+ str(self.sub_title)

    def get_absolute_url(self):
        return reverse("shop:subcategory", kwargs={'slug': self.slug })


# Create your models here.
class ProductData(models.Model):
    subcategory = models.ForeignKey(SubCategory, related_name='products', on_delete=models.CASCADE)
    p_name = models.CharField(max_length=150, null=False, db_index=True)
    seller_name = models.CharField(max_length=70, null=True)
    p_owner = models.ForeignKey(User, on_delete=models.CASCADE, default=102, related_name='users_kisan')
    # p_owner_type = models.CharField(max_length=2, default='KP', editable=False)
    p_city = models.CharField(max_length=50, null=True)
    p_img = fields.ImageField(upload_to=get_product_location, null=True, blank=True)
    p_img2 = fields.ImageField(upload_to=get_product_location, null=True, blank=True)
    p_img3 = fields.ImageField(upload_to=get_product_location, null=True, blank=True)
    p_img4 = fields.ImageField(upload_to=get_product_location, null=True, blank=True)
    p_price = models.IntegerField()
    min_q = models.CharField(max_length=10)
    UNIT_CHOICES = (
        ('KGs', "Kilo Gram"),
        ('GMs', "Grams"),
        ('Quintal', "Quintal"),
        ('Tons', "Tons"),
        ('LTRs', "liter"),
        ('MLs', "Mililiter"),
        ('unit', "Unit"),
        ('Pack', "Pack"),
    )
    q_unit = models.CharField(max_length=10, choices=UNIT_CHOICES, default='KGs')
    p_desc = models.TextField(null=True)
    tag = models.CharField(max_length=20, null=True)
    validity = models.DateField(default=future_date)
    p_organic = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now=True)
    priority = models.PositiveIntegerField(null=True, validators=[MaxValueValidator(10), MinValueValidator(1)])

    def save(self, *args, **kwargs):
        x = self.p_img.name.split('.')
        if x[-1] == 'png' or x[-1] == 'PNG':
            png = Image.open(self.p_img)
            png.load()
            background = Image.new("RGB", png.size, (255, 255, 255))
            background.paste(png, mask=png.split()[3])
            new_im = BytesIO()
            background.save(new_im, 'JPEG', quality=80)
            new_image1 = File(new_im, name=self.p_img.name)
        else:
            new_image1 = self.p_img
            if self.p_img2:
                new_image2 = self.p_img2
                new_image2 = compress(new_image2)
                self.p_img2 = new_image2
            if self.p_img3:
                new_image3 = self.p_img3
                new_image3 = compress(new_image3)
                self.p_img3 = new_image3
            if self.p_img4:
                new_image4 = self.p_img4
                new_image4 = compress(new_image4)
                self.p_img4 = new_image4
        new_image = compress(new_image1)
        self.p_img = new_image
        super().save(*args, **kwargs)
    
    def __str__(self):
        return str(self.id) +" - "+str(self.subcategory) +" - "+ str(self.p_owner) +" - "+ str(self.p_name)
        
    def get_absolute_url(self):
        return reverse('product_detail', args=(self.id,))

    @property
    def get_products(self):
        return ProductData.objects.filter(subcategories__sub_title=self.title)


class UsersRated(models.Model):
    user = models.CharField(max_length=50, null=True)


class Rating(models.Model):
    users = models.ManyToManyField(UsersRated)
    seller = models.CharField(max_length=50, null=True)
    rate = models.IntegerField(default=0)


class link_mf_product(models.Model):
    product = models.ForeignKey(ProductData, related_name='mf_seller', on_delete=models.CASCADE)
    seller_id = models.IntegerField(default=0, max_length=5)
    seller_price = models.DecimalField(max_digits=10, decimal_places=2, blank=False, default=0)
    p_city = models.CharField(max_length=50, null=True)
    created_date = models.DateTimeField(auto_now=True)

    def __str__(self):
        return str(self.product_id) +" - "+ str(self.product.p_name) +". MF-ID- "+ str(self.seller_id)

# answers = link_seller_product.objects.filter(p_city='name', ProductData_p_name='name').select_related()
